//*****************************************************************************
// File Name	: a2dtest.c
// 
// Title		: example usage of some avr library functions
// Revision		: 1.0
// Notes		:	
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
// 
// Revision History:
// When			Who			Description of change
// -----------	-----------	-----------------------
// 20-Oct-2002	pstang		Created the program
//*****************************************************************************
 
//----- Include Files ---------------------------------------------------------
#include <avr/io.h>		// include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include interrupt support
#include <math.h>

#include "global.h"		// include our global settings
#include "uart2.h"		// include uart function library
#include "rprintf.h"	// include printf function library
#include "timer.h"		// include timer function library (timing, PWM, etc)
#include "a2d.h"		// include A/D converter function library

//----- Begin Code ------------------------------------------------------------
#define BUFLEN 64

int main(void)
{
	u16 i,x,y;
	double fi, err, fibuf[BUFLEN];
	s16 fia, erra;
	u16 fib, errb;

	// initialize our libraries
	// initialize the UART (serial port)
	uartInit();
	uartSetBaudRate(0,9600);
	// make all rprintf statements use uart for output
	rprintfInit(uart0SendByte);
	// initialize the timer system
	timerInit();
	// turn on and initialize A/D converter
	a2dInit();	
	// configure a2d port (PORTA) as input
	// so we can receive analog signals
	DDRA = 0x00;
	// make sure pull-up resistors are turned off
	PORTA = 0x00;

	// set the a2d prescaler (clock division ratio)
	// - a lower prescale setting will make the a2d converter go faster
	// - a higher setting will make it go slower but the measurements
	//   will be more accurate
	// - other allowed prescale values can be found in a2d.h
	a2dSetPrescaler(ADC_PRESCALE_DIV128);

	// set the a2d reference
	// - the reference is the voltage against which a2d measurements are made
	// - other allowed reference values can be found in a2d.h
	a2dSetReference(ADC_REFERENCE_AREF);

	// use a2dConvert8bit(channel#) to get an 8bit a2d reading
	// use a2dConvert10bit(channel#) to get a 10bit a2d reading

	while(1)
	{
		fi=0;
		err=0;
		for(i=0; i<BUFLEN; i++)
		{ 
			x = a2dConvert10bit(ADC_CH_ADC0);
			y = a2dConvert10bit(ADC_CH_ADC1);
			fibuf[i] = atan2((double)x-511,(double)y-511);		// record computed angles to buffer for post processing
		}
		for(i=0; i<BUFLEN; i++) fi += fibuf[i];		// sum recorded angles

		fi = ((fi/BUFLEN)+PI) * 180.0 / PI;		// average recorded angles and convert product to degrees

		for(i=0; i<BUFLEN; i++) err += (fibuf[i]-fi)*(fibuf[i]-fi);		// sum cubic errors
		err = sqrt(err/(BUFLEN-1))/sqrt(BUFLEN);	// compute average cubic error
		erra = floor(err);
		errb = floor((err - erra)*1000);
		
		fia = floor(fi);
		fib = floor((fi - fia)*1000);

		
		rprintf("fi:%d.%d  +- %d.%d \r\n", fia, fib, erra, errb);
	}
	return 0;
}
